@@ -69,10 +69,13 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 
+#ifdef __APPLE__
+#include "compat/apple/net_compat.h"
+#endif
+
 #include "anet.h"
 
-static int open_fds = 0;
-static int max_fds = 0;
+#define _UNUSED(V) ((void) V)
 
 static void anetSetError(char *err, const char *fmt, ...)
 {
@@ -127,11 +130,20 @@ int anetSetSendBuffer(char *err, int fd, int buffsize)
 int anetTcpKeepAlive(char *err, int fd)
 {
     int yes = 1;
-    int idle = 15;
-    int interval = 5;
+
+    int idle = 20;
+    int interval = 2;
     int count = 3;
 
-    if (setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void*) &yes, sizeof(yes))
+    #ifdef __APPLE__
+    _UNUSED(yes);
+    if (set_tcp_keepalive(fd, idle, interval, count) == -1) {
+        anetSetError(err, "setsockopt tcp keepalive: %s", strerror(errno));
+        return ANET_ERR;
+    }
+    #else
+    if (
+            setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void*) &yes, sizeof(yes))
             || setsockopt(fd, SOL_TCP, TCP_KEEPIDLE, (void*) &idle, sizeof(idle))
             || setsockopt(fd, SOL_TCP, TCP_KEEPINTVL, (void*) &interval, sizeof(interval))
             || setsockopt(fd, SOL_TCP, TCP_KEEPCNT, (void*) &count, sizeof(count))
@@ -139,30 +151,28 @@ int anetTcpKeepAlive(char *err, int fd)
         anetSetError(err, "setsockopt SO_KEEPALIVE: %s", strerror(errno));
         return ANET_ERR;
     }
+    #endif
     return ANET_OK;
 }
 
-static int anetCreateSocket(char *err, int domain)
+static inline void emfileError() {
+    struct rlimit limits;
+    getrlimit(RLIMIT_NOFILE, &limits);
+    fprintf(stderr, "<3>EMFILE: Out of file descriptors (either there is a leak that needs fixing or you need to increase ulimit if you need more than %d connections)!\n", (int) limits.rlim_cur);
+}
+
+int anetCreateSocket(char *err, int domain, int typeFlags)
 {
     int s, on = 1;
-    if (!max_fds) {
-        struct rlimit limits;
-        getrlimit(RLIMIT_NOFILE, &limits);
-        max_fds = limits.rlim_cur - 20;
-        // maximum number of file descriptors we will use for sockets
-    }
-    if (open_fds >= max_fds) {
-        errno = EMFILE;
-        anetSetError(err, "approaching RLIMIT: %s", strerror(errno));
-        return ANET_ERR;
-    }
-    if ((s = socket(domain, SOCK_STREAM, 0)) == -1) {
+
+    if ((s = socket(domain, SOCK_STREAM | typeFlags, 0)) == -1) {
+        if (errno == EMFILE) {
+            emfileError();
+        }
         anetSetError(err, "creating socket: %s", strerror(errno));
         return ANET_ERR;
     }
 
-    open_fds++;
-
     /* Make sure connection-intensive things like the redis benckmark
      * will be able to close/open sockets a zillion of times */
     if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (void*)&on, sizeof(on)) == -1) {
@@ -198,14 +208,10 @@ static int anetTcpGenericConnect(char *err, char *addr, char *service, int flags
     }
 
     for (p = gai_result; p != NULL; p = p->ai_next) {
-        if ((s = anetCreateSocket(err, p->ai_family)) == ANET_ERR)
+        int nonBlock = (flags & ANET_CONNECT_NONBLOCK) ? SOCK_NONBLOCK : 0;
+        if ((s = anetCreateSocket(err, p->ai_family, nonBlock)) == ANET_ERR)
             continue;
 
-        if (flags & ANET_CONNECT_NONBLOCK) {
-            if (anetNonBlock(err,s) != ANET_OK)
-                return ANET_ERR;
-        }
-
         if (connect(s, p->ai_addr, p->ai_addrlen) >= 0 || (errno == EINPROGRESS && (flags & ANET_CONNECT_NONBLOCK))
            ) {
             // If we were passed a place to toss the sockaddr info, save it
@@ -266,14 +272,9 @@ int anetTcpNonBlockConnectAddr(char *err, struct addrinfo *p)
 {
     int s;
 
-    if ((s = anetCreateSocket(err, p->ai_family)) == ANET_ERR)
+    if ((s = anetCreateSocket(err, p->ai_family, SOCK_NONBLOCK)) == ANET_ERR)
         return ANET_ERR;
 
-    if (anetNonBlock(err,s) != ANET_OK) {
-        anetCloseSocket(s);
-        return ANET_ERR;
-    }
-
     if (connect(s, p->ai_addr, p->ai_addrlen) >= 0) {
         return s;
     }
@@ -292,8 +293,8 @@ int anetTcpNonBlockConnectAddr(char *err, struct addrinfo *p)
 int anetRead(int fd, char *buf, int count)
 {
     int nread, totlen = 0;
-    while(totlen != count) {
-        nread = read(fd,buf,count-totlen);
+    while(totlen < count) {
+        nread = read(fd, buf, (size_t) (count - totlen));
         if (nread == 0) return totlen;
         if (nread == -1) return -1;
         totlen += nread;
@@ -308,7 +309,7 @@ int anetWrite(int fd, char *buf, int count)
 {
     int nwritten, totlen = 0;
     while(totlen != count) {
-        nwritten = write(fd,buf,count-totlen);
+        nwritten = write(fd, buf, (size_t) (count - totlen));
         if (nwritten == 0) return totlen;
         if (nwritten == -1) return -1;
         totlen += nwritten;
@@ -329,10 +330,8 @@ static int anetListen(char *err, int s, struct sockaddr *sa, socklen_t len) {
         return ANET_ERR;
     }
 
-    /* Use a backlog of 512 entries. We pass 511 to the listen() call because
-     * the kernel does: backlogsize = roundup_pow_of_two(backlogsize + 1);
-     * which will thus give us a backlog of 512 entries */
-    if (listen(s, 511) == -1) {
+    // no real drawback to using a large backlog, will usually be capped to 4096 by the kernel
+    if (listen(s, 65535) == -1) {
         anetSetError(err, "listen: %s", strerror(errno));
         anetCloseSocket(s);
         return ANET_ERR;
@@ -340,7 +339,17 @@ static int anetListen(char *err, int s, struct sockaddr *sa, socklen_t len) {
     return ANET_OK;
 }
 
-int anetTcpServer(char *err, char *service, char *bindaddr, int *fds, int nfds)
+static void anetSetBuffers(int fd, int sndsize, int rcvsize) {
+    if (sndsize > 0 && setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (void*)&sndsize, sizeof(sndsize)) == -1) {
+        fprintf(stderr, "setsockopt SO_SNDBUF: %s", strerror(errno));
+    }
+
+    if (rcvsize > 0 && setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (void*)&rcvsize, sizeof(rcvsize)) == -1) {
+        fprintf(stderr, "setsockopt SO_RCVBUF: %s", strerror(errno));
+    }
+}
+
+int anetTcpServer(char *err, char *service, char *bindaddr, int *fds, int nfds, int flags, int sndsize, int rcvsize)
 {
     int s;
     int i = 0;
@@ -364,9 +373,11 @@ int anetTcpServer(char *err, char *service, char *bindaddr, int *fds, int nfds)
     }
 
     for (p = gai_result; p != NULL && i < nfds; p = p->ai_next) {
-        if ((s = anetCreateSocket(err, p->ai_family)) == ANET_ERR)
+        if ((s = anetCreateSocket(err, p->ai_family, flags)) == ANET_ERR)
             continue;
 
+        anetSetBuffers(s, sndsize, rcvsize);
+
         if (anetListen(err, s, p->ai_addr, p->ai_addrlen) == ANET_ERR) {
             continue;
         }
@@ -380,43 +391,57 @@ int anetTcpServer(char *err, char *service, char *bindaddr, int *fds, int nfds)
     }
     return (i > 0 ? i : ANET_ERR);
 }
-
-int anetGenericAccept(char *err, int s, struct sockaddr *sa, socklen_t *len)
+int anetUnixSocket(char *err, char *path, int flags)
 {
-    int fd;
-
-    if (!max_fds) {
-        struct rlimit limits;
-        getrlimit(RLIMIT_NOFILE, &limits);
-        max_fds = limits.rlim_cur - 20;
-        // maximum number of file descriptors we will use for sockets
-    }
-    if (open_fds >= max_fds) {
-        // accept and immediately close all pending connections
-        while ((fd = accept(s,sa,len)) >= 0) {
-            open_fds++;
-            anetCloseSocket(fd);
-        }
-        errno = EMFILE;
-        anetSetError(err, "approaching RLIMIT: %s", strerror(errno));
+    int s;
+    if ((s = anetCreateSocket(err, AF_UNIX, flags)) == ANET_ERR) {
         return ANET_ERR;
     }
 
-    fd = accept(s,sa,len);
+    struct sockaddr_un addr;
+    memset(&addr, 0, sizeof(addr));
+    addr.sun_family = AF_UNIX;
+    strcpy(addr.sun_path, path);
+
+    if (bind(s, (struct sockaddr*)&addr, sizeof(addr)) == -1) {
+        anetSetError(err, "bind: %s", strerror(errno));
+        anetCloseSocket(s);
+        return ANET_ERR;
+    }
+
+    // explicitely setting tcp buffers causes failure of linux tcp window auto tuning ... it just doesn't work well without the auto tuning
+    // anetSetBuffers(s, 512 * 1024, 64 * 1024);
+
+    // no real drawback to using a large backlog, will usually be capped to 4096 by the kernel
+    if (listen(s, 65535) == -1) {
+        anetSetError(err, "listen: %s", strerror(errno));
+        anetCloseSocket(s);
+        return ANET_ERR;
+    }
+
+    return s;
+}
+
+int anetGenericAccept(char *err, int s, struct sockaddr *sa, socklen_t *len, int flags)
+{
+    int fd;
+
+    fd = accept4(s, sa, len, flags);
 
     if (fd == -1) {
+        if (errno == EMFILE) {
+            emfileError();
+        }
         if (errno != EINTR) {
             anetSetError(err, "Generic accept error: %s %d", strerror(errno), fd);
         }
         return ANET_ERR;
     }
 
-    open_fds++;
-
     return fd;
 }
 
-static int get_socket_error(int fd) {
+static inline int get_socket_error(int fd) {
     int err = 1;
     socklen_t len = sizeof err;
     if (-1 == getsockopt(fd, SOL_SOCKET, SO_ERROR, (char *) &err, &len)) {
@@ -429,14 +454,14 @@ static int get_socket_error(int fd) {
 }
 
 void anetCloseSocket(int fd) {
-    if (fd >= 0) {
-        get_socket_error(fd); // First clear any errors, which can cause close to fail
-        if (shutdown(fd, SHUT_RDWR) < 0) { // Secondly, terminate the reliable delivery
-            if (errno != ENOTCONN && errno != EINVAL) { // SGI causes EINVAL
-                fprintf(stderr, "Shutdown client socket failed.\n");
-            }
-        }
-        close(fd); // Finally call anetCloseSocket() socket
-        open_fds--;
+    if (fd < 0) {
+        return;
     }
+    get_socket_error(fd); // First clear any errors, which can cause close to fail
+    if (shutdown(fd, SHUT_RDWR) < 0) { // Secondly, terminate the reliable delivery
+        if (errno != ENOTCONN && errno != EINVAL) { // SGI causes EINVAL
+            fprintf(stderr, "Shutdown client socket failed.\n");
+        }
+    }
+    close(fd);
 }
