@@ -35,6 +35,7 @@ static int decodeBDS30(struct modesMessage *mm, bool store);
 static int decodeBDS40(struct modesMessage *mm, bool store);
 static int decodeBDS50(struct modesMessage *mm, bool store);
 static int decodeBDS60(struct modesMessage *mm, bool store);
+static int decodeBDS44(struct modesMessage *mm, bool store);
 
 static CommBDecoderFn comm_b_decoders[] = {
     &decodeEmptyResponse,
@@ -44,7 +45,8 @@ static CommBDecoderFn comm_b_decoders[] = {
     &decodeBDS17,
     &decodeBDS40,
     &decodeBDS50,
-    &decodeBDS60
+    &decodeBDS60,
+    &decodeBDS44
 };
 
 void decodeCommB(struct modesMessage *mm) {
@@ -226,14 +228,21 @@ static int decodeBDS20(struct modesMessage *mm, bool store) {
     // score based on number of valid characters
     int score = 8;
     int valid = 1;
-    for (unsigned i = 0; i < 8; ++i) {
-        if ((callsign[i] >= 'A' && callsign[i] <= 'Z') || (callsign[i] >= '0' && callsign[i] <= '9') || callsign[i] == ' ') {
+    for (int i = 0; i < 8; ++i) {
+        if (
+                (callsign[i] >= 'A' && callsign[i] <= 'Z')
+                // -./0123456789
+                || (callsign[i] >= '-' && callsign[i] <= '9')
+                || callsign[i] == ' '
+                || callsign[i] == '@'
+           ) {
+            // valid chars
             score += 6;
-        } else if (callsign[i] == '@') {
-            // Padding (sometimes we get @@@@@@@@, i.e. BDS2,0 with all zeros - we do want to accept this as a BDS2,0 but not actually use the callsign)
-            valid = 0;
         } else {
             // Invalid
+            if (Modes.debug_callsign) {
+                fprintf(stderr, "%06x %8s (len: %d)\n", mm->addr, callsign, (int) strlen(callsign));
+            }
             return 0;
         }
     }
@@ -249,6 +258,76 @@ static int decodeBDS20(struct modesMessage *mm, bool store) {
     return score;
 }
 
+// check if the payload is a valid ACAS payload
+// https://mode-s.org/decode/book-the_1090mhz_riddle-junzi_sun.pdf
+int checkAcasRaValid(unsigned char *msg, struct modesMessage *mm, int debug) {
+
+    bool ara = getbit(msg, 9);
+    bool rat = getbit(msg, 27);
+    bool mte = getbit(msg, 28);
+
+    // not a valid RA if none of the bits are set
+    if (!ara && !rat && !mte)
+        return 0;
+
+    if (getbits(msg, 9, 28) == 0)
+        return 0; // these are the bits that contain the info, all zero it's not an RA
+    if (getbit(msg, 23) && getbit(msg, 24))
+        return 0; // complementary bits, both set is invalid (above / below)
+    if (getbit(msg, 25) && getbit(msg, 26))
+        return 0; // complementary bits, both set is invalid (left / right)
+
+    if (mm->msgtype == 16) {
+        if (getbits(msg, 29, 56) != 0)
+            return 0; // in DF16 messages msg bits 29 to 56 are reserved
+
+        return 1;
+    }
+
+    // some extra restrictions for DF != 16 below
+
+    if (getbit(msg, 25) || getbit(msg, 26))
+        return 0; // left / right isn't used, require zero
+
+    // for COMMB messages let's check if the thread indicator makes sense
+
+    int tti = getbits(msg, 29, 30);
+    // thread type indicator
+    // 00 No identity data in threat identity data
+    if (tti == 0) {
+        if (getbits(msg, 31, 56) != 0)
+            return 0;
+
+        return 1;
+    }
+    // When the threat type indicator is 01 , MB bits 31-54 contain the 24-bit Mode S transponder address and the last two bits are set to zero.
+    if (tti == 1) {
+        if (getbits(msg, 55, 56) != 0)
+            return 0;
+
+        uint32_t addr = getbits(msg, 31, 54);
+        if (icaoFilterTest(addr))
+            return 1;
+
+        return debug; // zero unless debug
+    }
+
+    // 10 Threat identity data contains altitude, range, and bearing
+    if (tti == 2) {
+        if (mm->metype == 28)// allow for DF17
+            return 1;
+        // hard to tell if used and separate from garbage, don't mark valid for the moment
+        return debug; // zero unless debug
+    }
+
+    // 11 Not assigned
+    if (tti == 3) {
+        return 0;
+    }
+
+    return 0;
+}
+
 // BDS3,0 ACAS RA
 
 static int decodeBDS30(struct modesMessage *mm, bool store) {
@@ -261,6 +340,7 @@ static int decodeBDS30(struct modesMessage *mm, bool store) {
 
     if (store) {
         mm->commb_format = COMMB_ACAS_RA;
+        mm->acas_ra_valid = 1;
     }
 
     // just accept it.
@@ -402,9 +482,9 @@ static int decodeBDS40(struct modesMessage *mm, bool store) {
         if (mode_valid) {
             mm->nav.modes_valid = 1;
             mm->nav.modes =
-                    ((mode_raw & 4) ? NAV_MODE_VNAV : 0) |
-                    ((mode_raw & 2) ? NAV_MODE_ALT_HOLD : 0) |
-                    ((mode_raw & 1) ? NAV_MODE_APPROACH : 0);
+                ((mode_raw & 4) ? NAV_MODE_VNAV : 0) |
+                ((mode_raw & 2) ? NAV_MODE_ALT_HOLD : 0) |
+                ((mode_raw & 1) ? NAV_MODE_APPROACH : 0);
         }
 
         if (source_valid) {
@@ -742,3 +822,140 @@ static int decodeBDS60(struct modesMessage *mm, bool store) {
 
     return score;
 }
+
+// BDS 4,4 Meteorological routine air report
+
+static int decodeBDS44(struct modesMessage *mm, bool store) {
+    unsigned char *msg = mm->MB;
+
+    unsigned source = getbits(msg, 1, 4);
+
+    unsigned wind_valid = getbit(msg, 5);
+    unsigned wind_speed_raw = getbits(msg, 6, 14);
+    unsigned wind_direction_raw = getbits(msg, 15, 23);
+
+    unsigned temperature_sign = getbit(msg, 24);
+    unsigned static_air_temperature_raw = getbits(msg, 25, 34);
+
+    unsigned pressure_valid = getbit(msg, 35);
+    unsigned static_pressure_raw = getbits(msg, 36, 46);
+
+    unsigned turbulence_valid = getbit(msg, 47);
+    unsigned turbulence_raw = getbits(msg, 48, 49);
+
+    unsigned humidity_valid = getbit(msg, 50);
+    unsigned humidity_raw = getbits(msg, 51, 56);
+
+    /*
+    if (!wind_valid || !temperature_valid || !pressure_valid || !turbulence_valid && !humidity_valid){
+       return 0;
+    }
+    */
+    int met_source = source;
+    int score = 0;
+    int wind_speed = 0;
+    float wind_direction = 0;
+    float temperature = 0;
+    int static_pressure = 0;
+    int turbulence = 0;
+    float humidity = 0;
+    if (met_source >= 0 && met_source <= 6) {
+        score += 4;
+    }
+    else {
+        return 0;
+    }
+    if (wind_valid){
+        wind_speed = (int)wind_speed_raw;
+        if (wind_speed <= 511 && wind_speed >= 0){
+            score += 9;
+        }
+        else {
+            return 0;
+        }
+        wind_direction = wind_direction_raw * (180 / 256);
+        if (wind_direction >= 0 && wind_direction <= 360){
+            score += 9;
+        }
+        else {
+            return 0;
+        }
+    }
+    else if (wind_speed == 0) {
+        score += 2;
+    }
+    if (temperature_sign){
+        temperature = (static_air_temperature_raw - pow(2, 10)) * 0.25;
+    }
+    else {
+        temperature = static_air_temperature_raw * 0.25;
+    }
+    if (temperature >= -128 && temperature <= 128){
+        score += 10;
+    }
+    else {
+        return 0;
+    }
+    if (pressure_valid){
+        static_pressure = (int)static_pressure_raw;
+        if (static_pressure >= 0 && static_pressure <= 2048){
+            score += 11;
+            return 0;
+        }
+        else {
+        }
+    }
+    else if (static_pressure == 0) {
+        score += 1;
+    }
+    if (turbulence_valid){
+        turbulence = (int)turbulence_raw;
+        if (turbulence >= 0 && turbulence <= 3) {
+            score += 2;
+        }
+        else {
+            return 0;
+        }
+    }
+    else if (turbulence == 0) {
+        score += 1;
+    }
+    if (humidity_valid) {
+        humidity = humidity_raw * (100.0f / 64);
+        if (humidity >= 0 && humidity <= 100){
+            score += 6;
+        }
+        else {
+            return 0;
+        }
+    }
+    else if (humidity == 0) {
+        score += 1;
+    }
+    if (store) {
+        mm->commb_format = COMMB_METEOROLOGICAL_ROUTINE;
+        mm->met_source_valid = 1;
+        mm->met_source = met_source;
+        if (wind_valid) { 
+            mm->wind_valid = 1;
+            mm->wind_speed = wind_speed;
+            mm->wind_direction = wind_direction;
+        }
+        mm->oat_valid = 1;
+        mm->oat = temperature;
+        if (pressure_valid) {
+            mm->static_pressure_valid = 1;
+            mm->static_pressure = static_pressure;
+        }
+        if (turbulence_valid) {
+            mm->turbulence_valid = 1;
+            mm->turbulence = turbulence;
+        }
+        if (humidity_valid) {
+            mm->humidity_valid = 1;
+            mm->humidity = humidity;
+        }
+    }
+    return score;
+}
+
