@@ -45,24 +45,216 @@
 // nb: the correlation functions sum to zero, so we do not need to adjust for the DC offset in the input signal
 // (adding any constant value to all of m[0..3] does not change the result)
 
-static inline int slice_phase0(uint16_t *m) {
-    return 5 * m[0] - 3 * m[1] - 2 * m[2];
+
+// Changes 2020 by wiedehopf:
+// 20 units per sample, 24 units per symbol that are distributed according to phase
+// 1 bit has 2 symbols, in a bit representing a one the first symbol is high and the second is low
+
+// The previous assumption was that symbols beyond our control are zero.
+// Let's make the assumption that the symbols beyond our control are a statistical mean of 0 and 1.
+// Such a mean is represented by 12 units per symbol.
+// As an example for the above let's discuss the first slice function:
+// Samples 0 and 1 are completely occupied by the bit we are trying to judge thus no outside symbols.
+// The 3rd sample is 8 units of our bit and 12 units of the following symbol.
+// Our bit contributes part of a low symbol represented by -8 units
+// but we also get 12 units of 0.5 resulting in +6 units from the following symbol.
+//
+// The above comment is how these changes started out, i'll leave them here as food for thought.
+// Using --ifile the coefficients from the above thought process were iteratively tweaked by hand.
+// Note one of the correlation functions is no longer DC balanced (but just slightly)
+// Further testing on your own samples using --ifile --quiet --stats is welcome
+// Note you might need to use --throttle unless your using wiedehopf's readsb fork,
+// otherwise position stats won't work as they rely on realtime differences between
+// reception of CPRs.
+// Creating a 5 minute sample with a gain of 43.9:
+// timeout 300 rtl_sdr -f 1090000000 -s 2400000 -g 43.9 sample.dat
+// Checking a set of correlation functions using the above sample:
+// make && ./readsb --device-type ifile --ifile sample.dat --quiet --stats
+
+static inline __attribute__((always_inline)) int slice_phase0(uint16_t *m) {
+    return 18 * m[0] - 15 * m[1] - 3 * m[2];
 }
 
-static inline int slice_phase1(uint16_t *m) {
-    return 4 * m[0] - m[1] - 3 * m[2];
+static inline __attribute__((always_inline)) int slice_phase1(uint16_t *m) {
+    return 14 * m[0] - 5 * m[1] - 9 * m[2];
 }
 
-static inline int slice_phase2(uint16_t *m) {
-    return 3 * m[0] + m[1] - 4 * m[2];
+// slightly DC unbalanced but better results
+static inline __attribute__((always_inline)) int slice_phase2(uint16_t *m) {
+    return 16 * m[0] + 5 * m[1] - 20 * m[2];
 }
 
-static inline int slice_phase3(uint16_t *m) {
-    return 2 * m[0] + 3 * m[1] - 5 * m[2];
+static inline __attribute__((always_inline)) int slice_phase3(uint16_t *m) {
+    return 7 * m[0] + 11 * m[1] - 18 * m[2];
 }
 
-static inline int slice_phase4(uint16_t *m) {
-    return m[0] + 5 * m[1] - 5 * m[2] - m[3];
+static inline __attribute__((always_inline)) int slice_phase4(uint16_t *m) {
+    return 4 * m[0] + 15 * m[1] - 20 * m[2] + 1 * m[3];
+}
+
+static uint32_t valid_df_short_bitset;        // set of acceptable DF values for short messages
+static uint32_t valid_df_long_bitset;         // set of acceptable DF values for long messages
+
+static uint32_t generate_damage_set(uint8_t df, unsigned damage_bits)
+{
+    uint32_t result = (1 << df);
+    if (!damage_bits)
+        return result;
+
+    for (unsigned bit = 0; bit < 5; ++bit) {
+        unsigned damaged_df = df ^ (1 << bit);
+        result |= generate_damage_set(damaged_df, damage_bits - 1);
+    }
+
+    return result;
+}
+
+static void init_bitsets()
+{
+    // DFs that we directly understand without correction
+    valid_df_short_bitset = (1 << 0) | (1 << 4) | (1 << 5) | (1 << 11);
+    valid_df_long_bitset = (1 << 16) | (1 << 17) | (1 << 18) | (1 << 20) | (1 << 21);
+
+#ifdef ENABLE_DF24
+    if (1)
+        valid_df_long_bitset |= (1 << 24) | (1 << 25) | (1 << 26) | (1 << 27) | (1 << 28) | (1 << 29) | (1 << 30) | (1 << 31);
+#endif
+
+    // if we can also repair DF damage, include those corrections
+    if (Modes.fixDF && Modes.nfix_crc) {
+        // only correct for possible DF17, other types are less useful usually (DF11/18 would also be possible)
+        valid_df_long_bitset |= generate_damage_set(17, 1);
+    }
+}
+
+
+// extract one byte from the mag buffers using slice_phase functions
+// advance pPtr and phase
+static inline __attribute__((always_inline)) uint8_t slice_byte(uint16_t **pPtr, int *phase) {
+    uint8_t theByte = 0;
+
+    switch (*phase) {
+        case 0:
+            theByte =
+                (slice_phase0(*pPtr) > 0 ? 0x80 : 0) |
+                (slice_phase2(*pPtr+2) > 0 ? 0x40 : 0) |
+                (slice_phase4(*pPtr+4) > 0 ? 0x20 : 0) |
+                (slice_phase1(*pPtr+7) > 0 ? 0x10 : 0) |
+                (slice_phase3(*pPtr+9) > 0 ? 0x08 : 0) |
+                (slice_phase0(*pPtr+12) > 0 ? 0x04 : 0) |
+                (slice_phase2(*pPtr+14) > 0 ? 0x02 : 0) |
+                (slice_phase4(*pPtr+16) > 0 ? 0x01 : 0);
+
+            *phase = 1;
+            *pPtr += 19;
+            break;
+
+        case 1:
+            theByte =
+                (slice_phase1(*pPtr) > 0 ? 0x80 : 0) |
+                (slice_phase3(*pPtr+2) > 0 ? 0x40 : 0) |
+                (slice_phase0(*pPtr+5) > 0 ? 0x20 : 0) |
+                (slice_phase2(*pPtr+7) > 0 ? 0x10 : 0) |
+                (slice_phase4(*pPtr+9) > 0 ? 0x08 : 0) |
+                (slice_phase1(*pPtr+12) > 0 ? 0x04 : 0) |
+                (slice_phase3(*pPtr+14) > 0 ? 0x02 : 0) |
+                (slice_phase0(*pPtr+17) > 0 ? 0x01 : 0);
+
+            *phase = 2;
+            *pPtr += 19;
+            break;
+
+        case 2:
+            theByte =
+                (slice_phase2(*pPtr) > 0 ? 0x80 : 0) |
+                (slice_phase4(*pPtr+2) > 0 ? 0x40 : 0) |
+                (slice_phase1(*pPtr+5) > 0 ? 0x20 : 0) |
+                (slice_phase3(*pPtr+7) > 0 ? 0x10 : 0) |
+                (slice_phase0(*pPtr+10) > 0 ? 0x08 : 0) |
+                (slice_phase2(*pPtr+12) > 0 ? 0x04 : 0) |
+                (slice_phase4(*pPtr+14) > 0 ? 0x02 : 0) |
+                (slice_phase1(*pPtr+17) > 0 ? 0x01 : 0);
+
+            *phase = 3;
+            *pPtr += 19;
+            break;
+
+        case 3:
+            theByte =
+                (slice_phase3(*pPtr) > 0 ? 0x80 : 0) |
+                (slice_phase0(*pPtr+3) > 0 ? 0x40 : 0) |
+                (slice_phase2(*pPtr+5) > 0 ? 0x20 : 0) |
+                (slice_phase4(*pPtr+7) > 0 ? 0x10 : 0) |
+                (slice_phase1(*pPtr+10) > 0 ? 0x08 : 0) |
+                (slice_phase3(*pPtr+12) > 0 ? 0x04 : 0) |
+                (slice_phase0(*pPtr+15) > 0 ? 0x02 : 0) |
+                (slice_phase2(*pPtr+17) > 0 ? 0x01 : 0);
+
+            *phase = 4;
+            *pPtr += 19;
+            break;
+
+        case 4:
+            theByte =
+                (slice_phase4(*pPtr) > 0 ? 0x80 : 0) |
+                (slice_phase1(*pPtr+3) > 0 ? 0x40 : 0) |
+                (slice_phase3(*pPtr+5) > 0 ? 0x20 : 0) |
+                (slice_phase0(*pPtr+8) > 0 ? 0x10 : 0) |
+                (slice_phase2(*pPtr+10) > 0 ? 0x08 : 0) |
+                (slice_phase4(*pPtr+12) > 0 ? 0x04 : 0) |
+                (slice_phase1(*pPtr+15) > 0 ? 0x02 : 0) |
+                (slice_phase3(*pPtr+17) > 0 ? 0x01 : 0);
+
+            *phase = 0;
+            *pPtr += 20;
+            break;
+    }
+    return theByte;
+}
+
+static void score_phase(int try_phase, uint16_t *pa, unsigned char **bestmsg, int *bestscore, int *bestphase, unsigned char **msg, unsigned char *msg1, unsigned char *msg2) {
+    Modes.stats_current.demod_preamblePhase[try_phase - 4]++;
+    uint16_t *pPtr;
+    int phase, score, bytelen;
+
+    pPtr = pa + 19 + (try_phase / 5);
+    phase = try_phase % 5;
+
+    (*msg)[0] = slice_byte(&pPtr, &phase);
+
+    // inspect DF field early, only continue processing
+    // messages where the DF appears valid
+    uint32_t df = ((uint8_t) (*msg)[0]) >> 3;
+    if (valid_df_long_bitset & (1 << df)) {
+        bytelen = MODES_LONG_MSG_BYTES;
+    } else if (valid_df_short_bitset & (1 << df)) {
+        bytelen = MODES_SHORT_MSG_BYTES;
+    } else {
+        score = -2;
+        if (score > *bestscore) {
+            // this is only for preamble stats
+            *bestscore = score;
+        }
+        return;
+    }
+
+    for (int i = 1; i < bytelen; ++i) {
+        (*msg)[i] = slice_byte(&pPtr, &phase);
+    }
+
+    // Score the mode S message and see if it's any good.
+    score = scoreModesMessage(*msg, bytelen * 8);
+    if (score > *bestscore) {
+        // new high score!
+        *bestmsg = *msg;
+        *bestscore = score;
+        *bestphase = try_phase;
+
+        // swap to using the other buffer so we don't clobber our demodulated data
+        // (if we find a better result then we'll swap back, but that's OK because
+        // we no longer need this copy if we found a better one)
+        *msg = (*msg == msg1) ? msg2 : msg1;
+    }
 }
 
 //
@@ -70,26 +262,43 @@ static inline int slice_phase4(uint16_t *m) {
 // try to demodulate some Mode S messages.
 //
 void demodulate2400(struct mag_buf *mag) {
-    static struct modesMessage zeroMessage;
-    struct modesMessage mm;
     unsigned char msg1[MODES_LONG_MSG_BYTES], msg2[MODES_LONG_MSG_BYTES], *msg;
-    uint32_t j;
 
-    unsigned char *bestmsg;
-    int bestscore, bestphase;
+    unsigned char *bestmsg = NULL;
+    int bestscore;
+    int bestphase = 0;
 
     uint16_t *m = mag->data;
     uint32_t mlen = mag->length;
 
     uint64_t sum_scaled_signal_power = 0;
 
+    // initialize bitsets on first call
+    if (!valid_df_short_bitset)
+        init_bitsets();
+
     msg = msg1;
 
-    for (j = 0; j < mlen; j++) {
-        uint16_t *preamble = &m[j];
-        int high;
-        uint32_t base_signal, base_noise;
-        int try_phase;
+    // advance ifile artificial clock even if we don't receive anything
+    if (Modes.sdr_type == SDR_IFILE && Modes.synthetic_now) {
+        Modes.synthetic_now = mag->sysTimestamp;
+    }
+
+    uint16_t *pa = m;
+    uint16_t *stop = m + mlen;
+
+    uint16_t *statsProgress = m;
+
+    const uint32_t statsWindow = MODES_SHORT_MSG_SAMPLES / 2; // half a short message
+    uint32_t loudEvents = 0;
+    uint32_t noiseLowSamples = 0;
+    uint32_t noiseHighSamples = 0;
+    const uint32_t loudThreshold = Modes.loudThreshold * Modes.loudThreshold * statsWindow;
+    const uint32_t noiseLowThreshold = Modes.noiseLowThreshold * Modes.noiseLowThreshold * statsWindow;
+    const uint32_t noiseHighThreshold = Modes.noiseHighThreshold * Modes.noiseHighThreshold * statsWindow;
+
+    for (; pa < stop; pa++) {
+        int32_t pa_mag, base_noise, ref_level;
         int msglen;
 
         // Look for a message starting at around sample 0 with phase offset 3..7
@@ -103,205 +312,105 @@ void demodulate2400(struct mag_buf *mag) {
         // phase 5: 0/5\1/3 3\0 0 0 0/3 3\1/5\0 0 0 0 0 0 0 X1
         // phase 6: 0/4\2 2/4\0 0 0 0 2/4\0/5\1 0 0 0 0 0 0 X2
         // phase 7: 0/3 3\1/5\0 0 0 0 1/5\0/4\2 0 0 0 0 0 0 X3
-        //
 
-        // quick check: we must have a rising edge 0->1 and a falling edge 12->13
-        if (!(preamble[0] < preamble[1] && preamble[12] > preamble[13]))
+        // do a pre-check to reduce CPU usage
+        // some silly unrolling that cuts CPU cycles
+        // due to plenty room in the message buffer for decoding
+        // we can with pa go beyond stop without a buffer overrun ...
+
+        if (Modes.autoGain && pa >= statsProgress) {
+            uint32_t magSum = 0;
+            for (uint32_t i = 0; i < statsWindow; i++) {
+                magSum += pa[i];
+            }
+            loudEvents += (magSum > loudThreshold);
+            noiseLowSamples += statsWindow * (magSum < noiseLowThreshold);
+            noiseHighSamples += statsWindow * (magSum < noiseHighThreshold);
+
+            statsProgress = pa + statsWindow;
+        }
+
+        if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+        pa++; if (pa[1] > pa[7] && pa[12] > pa[14] && pa[12] > pa[15]) { goto after_pre; }
+
+        continue;
+
+after_pre:
+        // ... but we must NOT decode if have ran past stop
+        if (!(pa < stop))
             continue;
 
-        if (preamble[1] > preamble[2] && // 1
-                preamble[2] < preamble[3] && preamble[3] > preamble[4] && // 3
-                preamble[8] < preamble[9] && preamble[9] > preamble[10] && // 9
-                preamble[10] < preamble[11]) { // 11-12
+        // 5 noise samples
+        base_noise = pa[5] + pa[8] + pa[16] + pa[17] + pa[18];
+        // pa_mag is the sum of the 4 preamble high bits
+        // minus 2 low bits between each of high bit pairs
+
+        // reduce number of preamble detections if we recently dropped samples
+        if (Modes.stats_15min.samples_dropped)
+            ref_level = base_noise * imax(PREAMBLE_THRESHOLD_PIZERO, Modes.preambleThreshold);
+        else
+            ref_level = base_noise * Modes.preambleThreshold;
+
+        ref_level >>= 5; // divide by 32
+
+        bestscore = -42;
+
+        int32_t diff_2_3 =  pa[2] - pa[3];
+        int32_t sum_1_4 = pa[1] + pa[4];
+        int32_t diff_10_11 = pa[10] - pa[11];
+        int32_t common3456 = sum_1_4 - diff_2_3 + pa[9] + pa[12];
+
+        // sample#: 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
+        // phase 3: 2/4\0/5\1 0 0 0 0/5\1/3 3\0 0 0 0 0 0 X4
+        // phase 4: 1/5\0/4\2 0 0 0 0/4\2 2/4\0 0 0 0 0 0 0 X0
+        pa_mag = common3456 - diff_10_11;
+        if (pa_mag >= ref_level) {
             // peaks at 1,3,9,11-12: phase 3
-            high = (preamble[1] + preamble[3] + preamble[9] + preamble[11] + preamble[12]) / 4;
-            base_signal = preamble[1] + preamble[3] + preamble[9];
-            base_noise = preamble[5] + preamble[6] + preamble[7];
-        } else if (preamble[1] > preamble[2] && // 1
-                preamble[2] < preamble[3] && preamble[3] > preamble[4] && // 3
-                preamble[8] < preamble[9] && preamble[9] > preamble[10] && // 9
-                preamble[11] < preamble[12]) { // 12
+            score_phase(4, pa, &bestmsg, &bestscore, &bestphase, &msg, msg1, msg2);
+
             // peaks at 1,3,9,12: phase 4
-            high = (preamble[1] + preamble[3] + preamble[9] + preamble[12]) / 4;
-            base_signal = preamble[1] + preamble[3] + preamble[9] + preamble[12];
-            base_noise = preamble[5] + preamble[6] + preamble[7] + preamble[8];
-        } else if (preamble[1] > preamble[2] && // 1
-                preamble[2] < preamble[3] && preamble[4] > preamble[5] && // 3-4
-                preamble[8] < preamble[9] && preamble[10] > preamble[11] && // 9-10
-                preamble[11] < preamble[12]) { // 12
+            score_phase(5, pa, &bestmsg, &bestscore, &bestphase, &msg, msg1, msg2);
+        }
+
+        // sample#: 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
+        // phase 5: 0/5\1/3 3\0 0 0 0/3 3\1/5\0 0 0 0 0 0 0 X1
+        // phase 6: 0/4\2 2/4\0 0 0 0 2/4\0/5\1 0 0 0 0 0 0 X2
+        pa_mag = common3456 + diff_10_11;
+        if (pa_mag >= ref_level) {
             // peaks at 1,3-4,9-10,12: phase 5
-            high = (preamble[1] + preamble[3] + preamble[4] + preamble[9] + preamble[10] + preamble[12]) / 4;
-            base_signal = preamble[1] + preamble[12];
-            base_noise = preamble[6] + preamble[7];
-        } else if (preamble[1] > preamble[2] && // 1
-                preamble[3] < preamble[4] && preamble[4] > preamble[5] && // 4
-                preamble[9] < preamble[10] && preamble[10] > preamble[11] && // 10
-                preamble[11] < preamble[12]) { // 12
+            score_phase(6, pa, &bestmsg, &bestscore, &bestphase, &msg, msg1, msg2);
+
             // peaks at 1,4,10,12: phase 6
-            high = (preamble[1] + preamble[4] + preamble[10] + preamble[12]) / 4;
-            base_signal = preamble[1] + preamble[4] + preamble[10] + preamble[12];
-            base_noise = preamble[5] + preamble[6] + preamble[7] + preamble[8];
-        } else if (preamble[2] > preamble[3] && // 1-2
-                preamble[3] < preamble[4] && preamble[4] > preamble[5] && // 4
-                preamble[9] < preamble[10] && preamble[10] > preamble[11] && // 10
-                preamble[11] < preamble[12]) { // 12
-            // peaks at 1-2,4,10,12: phase 7
-            high = (preamble[1] + preamble[2] + preamble[4] + preamble[10] + preamble[12]) / 4;
-            base_signal = preamble[4] + preamble[10] + preamble[12];
-            base_noise = preamble[6] + preamble[7] + preamble[8];
-        } else {
-            // no suitable peaks
-            continue;
+            score_phase(7, pa, &bestmsg, &bestscore, &bestphase, &msg, msg1, msg2);
         }
 
-        // Check for enough signal
-        if (base_signal * 2 < 3 * base_noise) // about 3.5dB SNR
+        // peaks at 1-2,4,10,12: phase 7
+        // sample#: 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
+        // phase 7: 0/3 3\1/5\0 0 0 0 1/5\0/4\2 0 0 0 0 0 0 X3
+        pa_mag = sum_1_4 + 2 * diff_2_3 + diff_10_11 + pa[12];
+        if (pa_mag >= ref_level)
+            score_phase(8, pa, &bestmsg, &bestscore, &bestphase, &msg, msg1, msg2);
+
+
+        // no preamble detected
+        if (bestscore == -42)
             continue;
 
-        // Check that the "quiet" bits 6,7,15,16,17 are actually quiet
-        if (preamble[5] >= high ||
-                preamble[6] >= high ||
-                preamble[7] >= high ||
-                preamble[8] >= high ||
-                preamble[14] >= high ||
-                preamble[15] >= high ||
-                preamble[16] >= high ||
-                preamble[17] >= high ||
-                preamble[18] >= high) {
-            continue;
-        }
-
-        // try all phases
+        // we had at least one phase greater than the preamble threshold
+        // and used scoremodesmessage on those bytes
         Modes.stats_current.demod_preambles++;
-        bestmsg = NULL;
-        bestscore = -2;
-        bestphase = -1;
-        for (try_phase = 4; try_phase <= 8; ++try_phase) {
-            uint16_t *pPtr;
-            int phase, i, score, bytelen;
-
-            // Decode all the next 112 bits, regardless of the actual message
-            // size. We'll check the actual message type later
-
-            pPtr = &m[j + 19] + (try_phase / 5);
-            phase = try_phase % 5;
-
-            bytelen = MODES_LONG_MSG_BYTES;
-            for (i = 0; i < bytelen; ++i) {
-                uint8_t theByte = 0;
-
-                switch (phase) {
-                    case 0:
-                        theByte =
-                                (slice_phase0(pPtr) > 0 ? 0x80 : 0) |
-                                (slice_phase2(pPtr + 2) > 0 ? 0x40 : 0) |
-                                (slice_phase4(pPtr + 4) > 0 ? 0x20 : 0) |
-                                (slice_phase1(pPtr + 7) > 0 ? 0x10 : 0) |
-                                (slice_phase3(pPtr + 9) > 0 ? 0x08 : 0) |
-                                (slice_phase0(pPtr + 12) > 0 ? 0x04 : 0) |
-                                (slice_phase2(pPtr + 14) > 0 ? 0x02 : 0) |
-                                (slice_phase4(pPtr + 16) > 0 ? 0x01 : 0);
-
-
-                        phase = 1;
-                        pPtr += 19;
-                        break;
-
-                    case 1:
-                        theByte =
-                                (slice_phase1(pPtr) > 0 ? 0x80 : 0) |
-                                (slice_phase3(pPtr + 2) > 0 ? 0x40 : 0) |
-                                (slice_phase0(pPtr + 5) > 0 ? 0x20 : 0) |
-                                (slice_phase2(pPtr + 7) > 0 ? 0x10 : 0) |
-                                (slice_phase4(pPtr + 9) > 0 ? 0x08 : 0) |
-                                (slice_phase1(pPtr + 12) > 0 ? 0x04 : 0) |
-                                (slice_phase3(pPtr + 14) > 0 ? 0x02 : 0) |
-                                (slice_phase0(pPtr + 17) > 0 ? 0x01 : 0);
-
-                        phase = 2;
-                        pPtr += 19;
-                        break;
-
-                    case 2:
-                        theByte =
-                                (slice_phase2(pPtr) > 0 ? 0x80 : 0) |
-                                (slice_phase4(pPtr + 2) > 0 ? 0x40 : 0) |
-                                (slice_phase1(pPtr + 5) > 0 ? 0x20 : 0) |
-                                (slice_phase3(pPtr + 7) > 0 ? 0x10 : 0) |
-                                (slice_phase0(pPtr + 10) > 0 ? 0x08 : 0) |
-                                (slice_phase2(pPtr + 12) > 0 ? 0x04 : 0) |
-                                (slice_phase4(pPtr + 14) > 0 ? 0x02 : 0) |
-                                (slice_phase1(pPtr + 17) > 0 ? 0x01 : 0);
-
-                        phase = 3;
-                        pPtr += 19;
-                        break;
-
-                    case 3:
-                        theByte =
-                                (slice_phase3(pPtr) > 0 ? 0x80 : 0) |
-                                (slice_phase0(pPtr + 3) > 0 ? 0x40 : 0) |
-                                (slice_phase2(pPtr + 5) > 0 ? 0x20 : 0) |
-                                (slice_phase4(pPtr + 7) > 0 ? 0x10 : 0) |
-                                (slice_phase1(pPtr + 10) > 0 ? 0x08 : 0) |
-                                (slice_phase3(pPtr + 12) > 0 ? 0x04 : 0) |
-                                (slice_phase0(pPtr + 15) > 0 ? 0x02 : 0) |
-                                (slice_phase2(pPtr + 17) > 0 ? 0x01 : 0);
-
-                        phase = 4;
-                        pPtr += 19;
-                        break;
-
-                    case 4:
-                        theByte =
-                                (slice_phase4(pPtr) > 0 ? 0x80 : 0) |
-                                (slice_phase1(pPtr + 3) > 0 ? 0x40 : 0) |
-                                (slice_phase3(pPtr + 5) > 0 ? 0x20 : 0) |
-                                (slice_phase0(pPtr + 8) > 0 ? 0x10 : 0) |
-                                (slice_phase2(pPtr + 10) > 0 ? 0x08 : 0) |
-                                (slice_phase4(pPtr + 12) > 0 ? 0x04 : 0) |
-                                (slice_phase1(pPtr + 15) > 0 ? 0x02 : 0) |
-                                (slice_phase3(pPtr + 17) > 0 ? 0x01 : 0);
-
-                        phase = 0;
-                        pPtr += 20;
-                        break;
-                }
-
-                msg[i] = theByte;
-                if (i == 0) {
-                    switch (msg[0] >> 3) {
-                        case 0: case 4: case 5: case 11:
-                            bytelen = MODES_SHORT_MSG_BYTES;
-                            break;
-
-                        case 16: case 17: case 18: case 20: case 21: case 24:
-                            break;
-
-                        default:
-                            bytelen = 1; // unknown DF, give up immediately
-                            break;
-                    }
-                }
-            }
-
-            // Score the mode S message and see if it's any good.
-            score = scoreModesMessage(msg, i * 8);
-            if (score > bestscore) {
-                // new high score!
-                bestmsg = msg;
-                bestscore = score;
-                bestphase = try_phase;
-
-                // swap to using the other buffer so we don't clobber our demodulated data
-                // (if we find a better result then we'll swap back, but that's OK because
-                // we no longer need this copy if we found a better one)
-                msg = (msg == msg1) ? msg2 : msg1;
-            }
-        }
 
         // Do we have a candidate?
         if (bestscore < 0) {
+
             if (bestscore == -1)
                 Modes.stats_current.demod_rejected_unknown_icao++;
             else
@@ -309,24 +418,29 @@ void demodulate2400(struct mag_buf *mag) {
             continue; // nope.
         }
 
-        msglen = modesMessageLenByType(bestmsg[0] >> 3);
+        msglen = modesMessageLenByType(getbits(bestmsg, 1, 5));
 
-        // Set initial mm structure details
-        mm = zeroMessage;
+        struct modesMessage *mm = netGetMM(&Modes.netMessageBuffer[0]);
 
         // For consistency with how the Beast / Radarcape does it,
         // we report the timestamp at the end of bit 56 (even if
         // the frame is a 112-bit frame)
-        mm.timestampMsg = mag->sampleTimestamp + j * 5 + (8 + 56) * 12 + bestphase;
+        mm->timestamp = mag->sampleTimestamp + (pa -m) * 5 + (8 + 56) * 12 + bestphase;
 
         // compute message receive time as block-start-time + difference in the 12MHz clock
-        mm.sysTimestampMsg = mag->sysTimestamp + receiveclock_ms_elapsed(mag->sampleTimestamp, mm.timestampMsg);
+        mm->sysTimestamp = mag->sysTimestamp + receiveclock_ms_elapsed(mag->sampleTimestamp, mm->timestamp);
 
-        mm.score = bestscore;
+        // advance ifile artifical clock for every message received
+        if (Modes.sdr_type == SDR_IFILE && Modes.synthetic_now) {
+            Modes.synthetic_now = mm->sysTimestamp;
+        }
+
+        mm->score = bestscore;
 
         // Decode the received message
         {
-            int result = decodeModesMessage(&mm, bestmsg);
+            memcpy(mm->msg, bestmsg, MODES_LONG_MSG_BYTES);
+            int result = decodeModesMessage(mm);
             if (result < 0) {
                 if (result == -1)
                     Modes.stats_current.demod_rejected_unknown_icao++;
@@ -334,10 +448,12 @@ void demodulate2400(struct mag_buf *mag) {
                     Modes.stats_current.demod_rejected_bad++;
                 continue;
             } else {
-                Modes.stats_current.demod_accepted[mm.correctedbits]++;
+                Modes.stats_current.demod_accepted[mm->correctedbits]++;
             }
         }
 
+        Modes.stats_current.demod_bestPhase[bestphase - 4]++;
+
         // measure signal power
         {
             double signal_power;
@@ -346,19 +462,19 @@ void demodulate2400(struct mag_buf *mag) {
             int k;
 
             for (k = 0; k < signal_len; ++k) {
-                uint32_t mag = m[j + 19 + k];
+                uint32_t mag = pa[19 + k];
                 scaled_signal_power += mag * mag;
             }
 
             signal_power = scaled_signal_power / 65535.0 / 65535.0;
-            mm.signalLevel = signal_power / signal_len;
+            mm->signalLevel = signal_power / signal_len;
             Modes.stats_current.signal_power_sum += signal_power;
             Modes.stats_current.signal_power_count += signal_len;
             sum_scaled_signal_power += scaled_signal_power;
 
-            if (mm.signalLevel > Modes.stats_current.peak_signal_power)
-                Modes.stats_current.peak_signal_power = mm.signalLevel;
-            if (mm.signalLevel > 0.50119)
+            if (mm->signalLevel > Modes.stats_current.peak_signal_power)
+                Modes.stats_current.peak_signal_power = mm->signalLevel;
+            if (mm->signalLevel > 0.50119)
                 Modes.stats_current.strong_signal_count++; // signal power above -3dBFS
         }
 
@@ -368,18 +484,27 @@ void demodulate2400(struct mag_buf *mag) {
         //  where the preamble of the second message clobbered the last
         //  few bits of the first message, but the message bits didn't
         //  overlap)
-        j += msglen * 12 / 5;
+        //pa += msglen * 12 / 5;
+        //
+        // let's test something, only jump part of the message and let the preamble detection handle the rest.
+        pa += msglen * 8 / 4;
 
         // Pass data to the next layer
-        useModesMessage(&mm);
+        netUseMessage(mm);
     }
 
+    mag->loudEvents = loudEvents;
+    mag->noiseLowSamples = noiseLowSamples;
+    mag->noiseHighSamples = noiseHighSamples;
+
     /* update noise power */
     {
         double sum_signal_power = sum_scaled_signal_power / 65535.0 / 65535.0;
         Modes.stats_current.noise_power_sum += (mag->mean_power * mag->length - sum_signal_power);
         Modes.stats_current.noise_power_count += mag->length;
     }
+
+    netDrainMessageBuffers();
 }
 
 
@@ -474,13 +599,10 @@ static void draw_modeac(uint16_t *m, unsigned modeac, unsigned f1_clock, unsigne
 //
 // one 2.4MHz sample = 25 cycles
 void demodulate2400AC(struct mag_buf *mag) {
-    struct modesMessage mm;
     uint16_t *m = mag->data;
     uint32_t mlen = mag->length;
     unsigned f1_sample;
 
-    memset(&mm, 0, sizeof (mm));
-
     double noise_stddev = sqrt(mag->mean_power - mag->mean_level * mag->mean_level); // Var(X) = E[(X-E[X])^2] = E[X^2] - (E[X])^2
     unsigned noise_level = (unsigned) ((mag->mean_power + noise_stddev) * 65535 + 0.5);
 
@@ -643,20 +765,23 @@ void demodulate2400AC(struct mag_buf *mag) {
 #endif
 
         // This message looks good, submit it
+        struct modesMessage *mm = netGetMM(&Modes.netMessageBuffer[0]);
 
         // For consistency with how the Beast / Radarcape does it,
         // we report the timestamp at the second framing pulse (F2)
-        mm.timestampMsg = mag->sampleTimestamp + f2_clock / 5; // 60MHz -> 12MHz
+        mm->timestamp = mag->sampleTimestamp + f2_clock / 5; // 60MHz -> 12MHz
 
         // compute message receive time as block-start-time + difference in the 12MHz clock
-        mm.sysTimestampMsg = mag->sysTimestamp + receiveclock_ms_elapsed(mag->sampleTimestamp, mm.timestampMsg);
+        mm->sysTimestamp = mag->sysTimestamp + receiveclock_ms_elapsed(mag->sampleTimestamp, mm->timestamp);
 
-        decodeModeAMessage(&mm, modeac);
+        decodeModeAMessage(mm, modeac);
 
         // Pass data to the next layer
-        useModesMessage(&mm);
+        netUseMessage(mm);
 
         f1_sample += (20 * 87 / 25);
         Modes.stats_current.demod_modeac++;
     }
+
+    netDrainMessageBuffers();
 }
