@@ -25,6 +25,13 @@
 #define DUMP1090_DEMOD_2400_H
 
 #include <stdint.h>
+#define PREAMBLE_THRESHOLD_MIN 40
+#define PREAMBLE_THRESHOLD_HOT 42
+#ifndef PREAMBLE_THRESHOLD_DEFAULT
+#define PREAMBLE_THRESHOLD_DEFAULT 58
+#endif
+#define PREAMBLE_THRESHOLD_PIZERO 75
+#define PREAMBLE_THRESHOLD_MAX 400
 
 struct mag_buf;
 
