@@ -31,6 +31,13 @@ void sdrInitConfig ();
 bool sdrHandleOption (int argc, char *argv);
 bool sdrOpen ();
 void sdrRun ();
+bool sdrHasRun();
+void sdrCancel ();
 void sdrClose ();
+void sdrSetGain (char *reason);
+
+void lockReader();
+void unlockReader();
+void wakeDecode();
 
 #endif
