@@ -2,7 +2,7 @@
 //
 // sdr_beast.c: Mode-S Beast and GNS5894 support
 //
-// Copyright (c) 2020 Michael Wolf <michael@mictronics.de>
+// Copyright (c) 2019 Michael Wolf <michael@mictronics.de>
 //
 // This file is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,14 +31,16 @@ static struct {
     uint16_t padding;
 } BeastSettings;
 
-static void beastSetOption(char opt) {
-    char optionsmsg[3] = {0x1a, '1', opt};
+static void beastSetOption(char opt)
+{
+    char optionsmsg[3] = { 0x1a, '1', opt };
     if (write(Modes.beast_fd, optionsmsg, 3) < 3) {
-        fprintf(stderr, "Beast failed to set option: %s\n", strerror(errno));
+        fprintf(stderr, "Beast failed to set option: %s", strerror(errno));
     }
 }
 
-void beastInitConfig(void) {
+void beastInitConfig(void)
+{
     free(Modes.beast_serial);
     Modes.beast_serial = strdup("/dev/ttyUSB0");
     BeastSettings.filter_df045 = false;
@@ -50,11 +52,62 @@ void beastInitConfig(void) {
     BeastSettings.crc = true;
 }
 
-bool beastHandleOption(int argc, char *argv) {
-    switch (argc) {
+static speed_t get_baud(int baud)
+{
+    switch (baud) {
+        case 9600:
+            return B9600;
+        case 19200:
+            return B19200;
+        case 38400:
+            return B38400;
+        case 57600:
+            return B57600;
+        case 115200:
+            return B115200;
+        case 230400:
+            return B230400;
+        case 460800:
+            return B460800;
+        case 500000:
+            return B500000;
+        case 576000:
+            return B576000;
+        case 921600:
+            return B921600;
+        case 921800:
+            // close enough
+            return B921600;
+        case 1000000:
+            return B1000000;
+        case 1152000:
+            return B1152000;
+        case 1500000:
+            return B1500000;
+        case 2000000:
+            return B2000000;
+        case 2500000:
+            return B2500000;
+        case 3000000:
+            return B3000000;
+        case 3500000:
+            return B3500000;
+        case 4000000:
+            return B4000000;
+        default:
+            return 0;
+    }
+}
+
+bool beastHandleOption(int key, char *arg)
+{
+    switch(key){
         case OptBeastSerial:
             free(Modes.beast_serial);
-            Modes.beast_serial = strdup(argv);
+            Modes.beast_serial = strdup(arg);
+            break;
+        case OptBeastBaudrate:
+            Modes.beast_baudrate = (int) strtoll(arg, NULL, 10);
             break;
         case OptBeastDF1117:
             BeastSettings.filter_df1117 = true;
@@ -76,6 +129,8 @@ bool beastHandleOption(int argc, char *argv) {
             Modes.mode_ac = 1;
             Modes.mode_ac_auto = 0;
             break;
+        default:
+            return false;
     }
     return true;
 }
@@ -84,7 +139,9 @@ bool beastOpen(void) {
     struct termios tios;
     speed_t baud = B3000000;
 
-    Modes.beast_fd = open(Modes.beast_serial, O_RDWR | O_NOCTTY);
+    int flags = O_RDWR | O_NOCTTY;
+    //flags |= O_NONBLOCK;
+    Modes.beast_fd = open(Modes.beast_serial, flags);
     if (Modes.beast_fd < 0) {
         fprintf(stderr, "Failed to open serial device %s: %s\n",
                 Modes.beast_serial, strerror(errno));
@@ -101,24 +158,39 @@ bool beastOpen(void) {
     tios.c_oflag = 0;
     tios.c_lflag = 0;
     tios.c_cflag = CS8 | CRTSCTS;
-    tios.c_cc[VMIN] = 11;
+    //tios.c_cc[VMIN] = 11; // read returns when a minimum of 11 characters are available
+    tios.c_cc[VMIN] = 0; // polling read with vtime 0
     tios.c_cc[VTIME] = 0;
 
     if (Modes.sdr_type == SDR_GNS) {
         baud = B921600;
     }
 
-    if (cfsetispeed(&tios, baud) < 0) {
-        fprintf(stderr, "Beast cfsetispeed(%s, %d): %s\n",
-                Modes.beast_serial, baud, strerror(errno));
-        return false;
+    if (Modes.beast_baudrate) {
+        baud = get_baud(Modes.beast_baudrate);
+        if (!baud) {
+            fprintf(stderr, "invalid baudrate: %d\n", Modes.beast_baudrate);
+            return false;
+        }
     }
 
-    if (cfsetospeed(&tios, baud) < 0) {
-        fprintf(stderr, "Beast cfsetospeed(%s, %d): %s\n",
-                Modes.beast_serial, baud, strerror(errno));
+#ifdef __APPLE__
+    if (cfsetspeed(&tios, baud) < 0) {
+
+        /* If standard rate setting fails, try IOKit's special baudrate setting */
+        if (ioctl(Modes.beast_fd, IOSSIOSPEED, &baud) < 0) {
+            fprintf(stderr, "Beast set speed(%s, %lu): %s\n",
+                    Modes.beast_serial, (unsigned long)baud, strerror(errno));
+            return false;
+        }
+    }
+#else
+    if (cfsetispeed(&tios, baud) < 0 || cfsetospeed(&tios, baud) < 0) {
+        fprintf(stderr, "Beast set speed(%s, %lu): %s\n",
+                Modes.beast_serial, (unsigned long)baud, strerror(errno));
         return false;
     }
+#endif
 
     tcflush(Modes.beast_fd, TCIFLUSH);
 
@@ -130,6 +202,7 @@ bool beastOpen(void) {
 
     if (Modes.sdr_type == SDR_MODESBEAST) {
         /* set options */
+        beastSetOption('B'); /* set classic beast mode */
         beastSetOption('C'); /* use binary format */
         beastSetOption('H'); /* RTS enabled */
 
@@ -174,21 +247,27 @@ bool beastOpen(void) {
 
     /* Kick on handshake and start reception */
     int RTSDTR_flag = TIOCM_RTS | TIOCM_DTR;
-    ioctl(Modes.beast_fd, TIOCMBIS, &RTSDTR_flag); //Set RTS&DTR pin
+    int res = ioctl(Modes.beast_fd, TIOCMBIS, &RTSDTR_flag); //Set RTS&DTR pin
+    if (res == -1) {
+        fprintf(stderr, "Serial device, reception start failed: %s\n", strerror(errno));
+        return false;
+    }
 
     if (Modes.sdr_type == SDR_MODESBEAST) {
-        fprintf(stderr, "Running Mode-S Beast via USB.\n");
+        fprintf(stderr, "Running Mode-S Beast via serial (over USB).\n");
     } else {
-        fprintf(stderr, "Running GNS HULC via USB.\n");
+        fprintf(stderr, "Running GNS HULC via serial (over USB).\n");
     }
     return true;
 }
 
-void beastRun() {
+void beastRun()
+{
 
 }
 
-void beastClose() {
+void beastClose()
+{
     /* Beast device will be closed in the main cleanup_and_exit function when
      * clients are freed.
      */
