@@ -48,22 +48,24 @@ void bladeRFInitConfig() {
     BladeRF.device = NULL;
 }
 
-bool bladeRFHandleOption(int argc, char *argv) {
-    switch (argc) {
+bool bladeRFHandleOption(int key, char *arg) {
+    switch (key) {
         case OptBladeFpgaDir:
-            BladeRF.fpga_path = strdup(argv);
+            BladeRF.fpga_path = strdup(arg);
             break;
         case OptBladeDecim:
-            BladeRF.decimation = atoi(argv);
+            BladeRF.decimation = atoi(arg);
             break;
         case OptBladeBw:
-            if (!strcasecmp(argv, "bypass")) {
+            if (!strcasecmp(arg, "bypass")) {
                 BladeRF.lpf_mode = BLADERF_LPF_BYPASSED;
             } else {
                 BladeRF.lpf_mode = BLADERF_LPF_NORMAL;
-                BladeRF.lpf_bandwidth = atoi(argv);
+                BladeRF.lpf_bandwidth = atoi(arg);
             }
             break;
+        default:
+            return false;
     }
     return true;
 }
@@ -289,18 +291,18 @@ static void *handle_bladerf_samples(struct bladerf *dev,
     static uint64_t nextTimestamp = 0;
     static bool dropping = false;
 
+    int64_t sysMicroseconds = mono_micro_seconds();
+    int64_t sysTimestamp = mstime();
+
     MODES_NOTUSED(dev);
     MODES_NOTUSED(stream);
     MODES_NOTUSED(meta);
     MODES_NOTUSED(user_data);
     MODES_NOTUSED(num_samples);
 
-    // record initial time for later sys timestamp calculation
-    uint64_t entryTimestamp = mstime();
-
-    pthread_mutex_lock(&Modes.data_mutex);
+    lockReader();
     if (Modes.exit) {
-        pthread_mutex_unlock(&Modes.data_mutex);
+        unlockReader();
         return BLADERF_STREAM_SHUTDOWN;
     }
 
@@ -309,15 +311,15 @@ static void *handle_bladerf_samples(struct bladerf *dev,
     struct mag_buf *lastbuf = &Modes.mag_buffers[(Modes.first_free_buffer + MODES_MAG_BUFFERS - 1) % MODES_MAG_BUFFERS];
     unsigned free_bufs = (Modes.first_filled_buffer - next_free_buffer + MODES_MAG_BUFFERS) % MODES_MAG_BUFFERS;
 
+    unlockReader();
+
     if (free_bufs == 0 || (dropping && free_bufs < MODES_MAG_BUFFERS / 2)) {
         // FIFO is full. Drop this block.
         dropping = true;
-        pthread_mutex_unlock(&Modes.data_mutex);
         return samples;
     }
 
     dropping = false;
-    pthread_mutex_unlock(&Modes.data_mutex);
 
     // Copy trailing data from last block (or reset if not valid)
     if (outbuf->dropped == 0) {
@@ -331,12 +333,15 @@ static void *handle_bladerf_samples(struct bladerf *dev,
     outbuf->length = 0;
     outbuf->mean_level = outbuf->mean_power = 0;
 
+    outbuf->sysTimestamp = sysTimestamp;
+    outbuf->sysMicroseconds = sysMicroseconds;
+
     unsigned blocks_processed = 0;
     unsigned samples_per_block = (BladeRF.block_size - 16) / 4;
 
     static bool overrun = true; // ignore initial overruns as we get up to speed
     static bool first_buffer = true;
-    for (unsigned offset = 0; offset < MODES_MAG_BUF_SAMPLES * 4; offset += BladeRF.block_size) {
+    for (unsigned offset = 0; offset < Modes.sdr_buf_samples * 4; offset += BladeRF.block_size) {
         // read the next metadata header
         uint8_t *header = ((uint8_t*) samples) + offset;
         uint64_t metadata_magic = le32toh(*(uint32_t*) (header));
@@ -397,14 +402,15 @@ static void *handle_bladerf_samples(struct bladerf *dev,
 
     if (blocks_processed) {
         // Get the approx system time for the start of this block
-        unsigned block_duration = 1e3 * outbuf->length / Modes.sample_rate;
-        outbuf->sysTimestamp = entryTimestamp - block_duration;
+        int64_t block_duration = 1e3 * outbuf->length / Modes.sample_rate;
+        outbuf->sysTimestamp -= block_duration;
+        outbuf->sysMicroseconds -= 1000 * block_duration;
 
         outbuf->mean_level /= blocks_processed;
         outbuf->mean_power /= blocks_processed;
 
         // Push the new data to the demodulation thread
-        pthread_mutex_lock(&Modes.data_mutex);
+        lockReader();
 
         // accumulate CPU while holding the mutex, and restart measurement
         end_cpu_timing(&thread_cpu, &Modes.reader_cpu_accumulator);
@@ -414,8 +420,8 @@ static void *handle_bladerf_samples(struct bladerf *dev,
         Modes.mag_buffers[next_free_buffer].length = 0; // just in case
         Modes.first_free_buffer = next_free_buffer;
 
-        pthread_cond_signal(&Modes.data_cond);
-        pthread_mutex_unlock(&Modes.data_mutex);
+        wakeDecode();
+        unlockReader();
     }
 
     return samples;
@@ -438,14 +444,14 @@ void bladeRFRun() {
             &buffers,
             /* num_buffers */ transfers,
             BLADERF_FORMAT_SC16_Q11_META,
-            /* samples_per_buffer */ MODES_MAG_BUF_SAMPLES,
+            /* samples_per_buffer */ Modes.sdr_buf_samples,
             /* num_transfers */ transfers,
             /* user_data */ NULL)) < 0) {
         fprintf(stderr, "bladerf_init_stream() failed: %s\n", bladerf_strerror(status));
         goto out;
     }
 
-    unsigned ms_per_transfer = 1000 * MODES_MAG_BUF_SAMPLES / Modes.sample_rate;
+    unsigned ms_per_transfer = 1000 * Modes.sdr_buf_samples / Modes.sample_rate;
     if ((status = bladerf_set_stream_timeout(BladeRF.device, BLADERF_MODULE_RX, ms_per_transfer * (transfers + 2))) < 0) {
         fprintf(stderr, "bladerf_set_stream_timeout() failed: %s\n", bladerf_strerror(status));
         goto out;
@@ -482,9 +488,8 @@ out:
 
 void bladeRFClose() {
     if (BladeRF.converter) {
-        cleanup_converter(BladeRF.converter_state);
+        cleanup_converter(&BladeRF.converter_state);
         BladeRF.converter = NULL;
-        BladeRF.converter_state = NULL;
     }
 
     if (BladeRF.device) {
