@@ -28,7 +28,9 @@
 void rtlsdrInitConfig ();
 bool rtlsdrOpen ();
 void rtlsdrRun ();
+void rtlsdrCancel();
 void rtlsdrClose ();
 bool rtlsdrHandleOption (int argc, char *argv);
+void rtlsdrSetGain(char *reason);
 
 #endif
